@@ -1,4 +1,5 @@
-# Copyright (c) 2010-2013 Trevor L. Davis <trevor.l.davis@stanford.edu>  
+# Copyright (c) 2010-2017 Trevor L. Davis <trevor.l.davis@gmail.com>  
+# Copyright (c) 2015 Rick FitzJohn https://github.com/richfitz
 # Copyright (c) 2013 Kirill Müller https://github.com/krlmlr
 # Copyright (c) 2011 Jim Nikelski <nikelski@bic.mni.mcgill.ca>
 # Copyright (c) 2010 Steve Lianoglou <lianos@cbio.mskcc.org> 
@@ -28,7 +29,7 @@
 
 #' Option Parser
 #' 
-#'  @slot usage The program usage message that will printed out if
+#' @slot usage The program usage message that will printed out if
 #'     \code{parse_args} finds a help option, \code{\%prog} is substituted with the
 #'     value of the \code{prog} argument.
 #' @slot options A list of of \code{OptionParserOption} instances that will
@@ -71,13 +72,14 @@ setClass("OptionParser", representation(usage = "character", options = "list",
 #'     by \code{parse_args} should \code{optparse} store option values.  Default is
 #'     derived from the long flag in \code{opt_str}.
 #' @slot default The default value \code{optparse} should use if it does not
-#'     find the option on the command line.  Default is derived from the long flag
-#'     in \code{opt_str}.
+#'     find the option on the command line.  
 #' @slot help A character string describing the option to be used by
 #'     \code{print_help} in generating a usage message.  \code{\%default} will be
 #'     substituted by the value of \code{default}.
 #' @slot metavar A character string that stands in for the option argument when
 #'     printing help text.  Default is the value of \code{dest}.
+#' @slot callback A function that executes after the each option value is fully parsed
+#' @slot callback_args Additional arguments that pass to the callback function. 
 #' @seealso \code{\link{make_option}}
 #' @exportClass OptionParserOption
 #' @export OptionParserOption
@@ -88,7 +90,9 @@ OptionParserOption <- setClass("OptionParserOption", representation(short_flag="
                                     dest="character",
                                     default="ANY",
                                     help="character",
-                                    metavar="character"),)
+                                    metavar="character",
+                                    callback="ANY",
+                                    callback_args="ANY"))
 
 #' A function to create an instance of a parser object
 #'
@@ -149,14 +153,14 @@ OptionParser <- function(usage = "usage: %prog [options]", option_list=list(),
                     description=description, epilogue=epilogue))
 }
 
-#'Functions to enable our OptionParser to recognize specific command line
-#'options.
+#' Functions to enable our OptionParser to recognize specific command line 
+#' options.
 #'
-#'\code{add_option} adds a option to a prexisting \code{OptionParser} instance
-#'whereas \code{make_option} is used to create a list of
-#'\code{OptionParserOption} instances that will be used in the
-#'\code{option_list} argument of the \code{OptionParser} function to create a
-#'new \code{OptionParser} instance.
+#' \code{add_option} adds a option to a prexisting \code{OptionParser} instance
+#' whereas \code{make_option} is used to create a list of
+#' \code{OptionParserOption} instances that will be used in the
+#' \code{option_list} argument of the \code{OptionParser} function to create a
+#' new \code{OptionParser} instance.
 #'
 #' @rdname add_make_option
 #' @param object An instance of the \code{OptionParser} class
@@ -182,13 +186,17 @@ OptionParser <- function(usage = "usage: %prog [options]", option_list=list(),
 #'     by \code{parse_args} should \code{optparse} store option values.  Default is
 #'     derived from the long flag in \code{opt_str}.
 #' @param default The default value \code{optparse} should use if it does not
-#'     find the option on the command line.  Default is derived from the long flag
-#'     in \code{opt_str}.
+#'     find the option on the command line.  
 #' @param help A character string describing the option to be used by
 #'     \code{print_help} in generating a usage message.  \code{\%default} will be
 #'     substituted by the value of \code{default}.
 #' @param metavar A character string that stands in for the option argument when
 #'     printing help text.  Default is the value of \code{dest}.
+#' @param callback A function that executes after the each option value is fully 
+#'     parsed.  It's value is assigned to the option and its arguments are
+#'     the option S4 object, the long flag string, the value of the option, 
+#'     the parser S4 object, and \code{...}.
+#' @param callback_args A list of additional arguments passed to callback function (via \code{do.call}).
 #' @return Both \code{make_option} and \code{add_option} return instances of
 #'     class \code{OptionParserOption}.
 #' @author Trevor Davis.
@@ -206,6 +214,8 @@ OptionParser <- function(usage = "usage: %prog [options]", option_list=list(),
 #'    # examples from package vignette
 #'    make_option(c("-v", "--verbose"), action="store_true", default=TRUE,
 #'        help="Print extra output [default]")
+#'    make_option(c("-q", "--quietly"), action="store_false",
+#'      dest="verbose", help="Print little output")
 #'    make_option(c("-c", "--count"), type="integer", default=5, 
 #'        help="Number of random normals to generate [default %default]",
 #'        metavar="number")
@@ -217,8 +227,8 @@ OptionParser <- function(usage = "usage: %prog [options]", option_list=list(),
 #'        help="Standard deviation if generator == \"rnorm\" [default %default]")
 #'
 #' @export
-make_option <- function(opt_str, action="store", type=NULL,
-                     dest=NULL, default=NULL, help="", metavar=NULL) {
+make_option <- function(opt_str, action="store", type=NULL, dest=NULL, default=NULL, help="", metavar=NULL, callback=NULL, callback_args=NULL) {
+
     # flags
     short_flag <- opt_str[grepl("^-[[:alpha:]]", opt_str)]
     if(length(short_flag)) {} else { short_flag <- as.character(NA) }
@@ -230,16 +240,16 @@ make_option <- function(opt_str, action="store", type=NULL,
         if( action %in% c("store_true", "store_false") ) {
             type <- "logical"
         }
-        else {
-            if( action %in% c("store") ) {
-                if (is.null(default)) {
-                    type <- "character"
-                } else {
-                    type <- typeof(default)
-                }
+        else if (action == "store") {
+            if (is.null(default)) {
+                type <- "character"
+            } else {
+                type <- typeof(default)
             }
         }
     }
+    if (is.null(type)) 
+        stop("Option type cannot be inferred. It or a default argument must be specified.")
     if (type == "numeric") { type <- "double" }
     # default
     if((type != typeof(default)) & !is.null(default)) {
@@ -255,37 +265,51 @@ make_option <- function(opt_str, action="store", type=NULL,
             metavar <- character(0)
         }
     }
-        
+    if (action == "callback") {            
+        if(!is.function(callback))
+            warning(sprintf("callback argument is not a function"))
+    } else {
+        if(!is.null(callback))      
+            warning(sprintf("callback argument is supplied for non-callback action")) 
+        if(!is.null(callback_args))      
+            warning(sprintf("callback_args argument is supplied for non-callback action")) 
+    }
+    if (is.null(callback_args))
+        callback_args <- list()
+       
     return(new("OptionParserOption", short_flag=short_flag, long_flag=long_flag,
                         action=action, type=type, dest=dest, default=default, 
-                        help=help, metavar=metavar))
+                        help=help, metavar=metavar, 
+                        callback=callback, callback_args=callback_args))
 }
+
 #' @rdname add_make_option
 #' @export
 add_option <- function(object, opt_str, action="store", type=NULL, 
-                    dest=NULL, default=NULL, help="", metavar=NULL) {
+                    dest=NULL, default=NULL, help="", metavar=NULL, 
+                    callback=NULL, callback_args=NULL) {
     options_list <- object@options
     n_original_options <- length(options_list)
     options_list[[n_original_options + 1]] <- make_option(opt_str=opt_str,
                                            action=action, type=type, dest=dest,
-                                           default=default, help=help, metavar=metavar)        
+                                           default=default, help=help, metavar=metavar,
+                                           callback=callback, callback_args=callback_args)        
     object@options <- options_list
     return(object)
 }
 
-#'Printing an usage message from an OptionParser object
+#' Printing an usage message from an OptionParser object
+#' 
+#' \code{print_help} print an usage message from an OptionParser object, usually
+#' called by \code{parse_args} when it encounters a help option.
 #'
-#'\code{print_help} print an usage message from an OptionParser object, usually
-#'called by \code{parse_args} when it encounters a help option.
+#' @param object A \code{OptionParser} instance.
+#' @return \code{print_help} uses the \code{cat} function to print out a usage
+#' message.  It returns \code{invisible(NULL)}.
+#' @author Trevor Davis.
 #'
-#'
-#'@param object A \code{OptionParser} instance.
-#'@return \code{print_help} uses the \code{cat} function to print out a usage
-#'message.  It returns \code{invisible(NULL)}.
-#'@author Trevor Davis.
-#'
-#'@seealso \code{{parse_args}} \code{{OptionParser}}
-#'@references Python's \code{optparse} library, which inspired this package,
+#' @seealso \code{{parse_args}} \code{{OptionParser}}
+#' @references Python's \code{optparse} library, which inspired this package,
 #'     is described here: \url{http://docs.python.org/library/optparse.html}
 #' @export
 print_help <- function(object) {
@@ -295,7 +319,7 @@ print_help <- function(object) {
     cat("Options:", sep="\n")    
 
     options_list <- object@options
-    for(ii in seq(along=options_list)) {
+    for(ii in seq_along(options_list)) {
         option <- options_list[[ii]]
         cat("\t")
         if(!is.na(option@short_flag)) {
@@ -323,6 +347,8 @@ print_help <- function(object) {
 .as_string <- function(default) {
     if(is.null(default)) {
         default_str <- "NULL"
+    } else if(!length(default)) {
+        default_str <- paste0(typeof(default), "(0)")
     } else if(is.na(default)) {
         default_str <- "NA"
     } else {
@@ -333,7 +359,9 @@ print_help <- function(object) {
 #' Parse command line options.
 #' 
 #' \code{parse_args} parses command line options using an \code{OptionParser}
-#' instance for guidance.
+#' instance for guidance. \code{parse_args2} is a wrapper to \code{parse_args}
+#' setting the options \code{positional_arguments} and \code{convert_hyphens_to_underscores}
+#' to \code{TRUE}.
 #' 
 #' @param object An \code{OptionParser} instance.
 #' @param args A character vector containing command line options to be parsed.
@@ -350,6 +378,9 @@ print_help <- function(object) {
 #'     \code{c(0, Inf)}.  The default \code{FALSE} is
 #'     supported for backward compatibility only, as it alters
 #'     the format of the return value.
+#' @param convert_hyphens_to_underscores If the names in the returned list of options
+#'      contains hyphens then convert them to underscores.  The default \code{FALSE} is 
+#'      supported for backward compatibility reasons as it alters the format of the return value
 #' @return Returns a list with field \code{options} containing our option values
 #'     as well as another field \code{args} which contains a vector of
 #'     positional arguments.  For backward compatibility, if and only if
@@ -390,25 +421,30 @@ print_help <- function(object) {
 #'
 #'# example from vignette using positional arguments
 #'option_list2 <- list( 
-#'    make_option(c("-n", "--add_numbers"), action="store_true", default=FALSE,
+#'    make_option(c("-n", "--add-numbers"), action="store_true", default=FALSE,
 #'        help="Print line number at the beginning of each line [default]")
 #'    )
 #'parser <- OptionParser(usage = "%prog [options] file", option_list=option_list2)
 #'
-#'parse_args(parser, args = c("--add_numbers", "example.txt"), positional_arguments = TRUE)
+#'parse_args(parser, args = c("--add-numbers", "example.txt"), positional_arguments = TRUE)
 #'
-#'parse_args(parser, args = c("-add_numbers", "example.txt"), positional_arguments = TRUE)
+#'parse_args(parser, args = c("--add-numbers", "example.txt"), positional_arguments = TRUE,
+#'          convert_hyphens_to_underscores = TRUE)
+#'
+#'parse_args2(parser, args = c("--add-numbers", "example.txt"))
 #'
 #' @import getopt
+#' @importFrom utils tail
 #' @export 
 parse_args <- function(object, args = commandArgs(trailingOnly = TRUE), 
-                    print_help_and_exit = TRUE, positional_arguments = FALSE) {
+                    print_help_and_exit = TRUE, positional_arguments = FALSE,
+                    convert_hyphens_to_underscores = FALSE) {
 
     n_options <- length( object@options )
 
     # Convert our option specification into ``getopt`` format
     spec <- matrix(NA, nrow = n_options, ncol = 5)
-    for (ii in seq(along = object@options)) {
+    for (ii in seq_along(object@options)) {
         spec[ii, ] <- .convert_to_getopt( object@options[[ii]] )
     }
 
@@ -453,12 +489,18 @@ parse_args <- function(object, args = commandArgs(trailingOnly = TRUE),
 
     options_list <- list()
     if(length(args)) {
-        opt <- getopt(spec=spec, opt=args)
+        opt <- try(getopt(spec=spec, opt=args), silent=TRUE)
+        if(class(opt) == "try-error") {
+            if(grepl("redundant short names for flags", opt)) {
+                opt <- paste(opt, "did you forget to set ``add_holp_option=FALSE`` in ``OptionParser``")
+            }
+            stop(opt)
+        }
     } else {
         opt <- list()
     }
 
-    for (ii in seq(along = object@options)) {
+    for (ii in seq_along(object@options)) {
         option <- object@options[[ii]]
         option_value <- opt[[sub("^--", "", option@long_flag)]] 
         if( !is.null(option_value) ) {
@@ -466,16 +508,33 @@ parse_args <- function(object, args = commandArgs(trailingOnly = TRUE),
                 options_list[[option@dest]] <- FALSE
             } else {    
                 options_list[[option@dest]] <- option_value
-            }
+            }         
+            if (option@action == "callback") {
+
+                callback_fn <- function(...) {
+                    option@callback(option, option@long_flag,  option_value, object, ...)
+                }
+                options_list[[option@dest]] <- do.call(callback_fn, option@callback_args)
+
+            } 
+
         } else {
             if( !is.null(option@default) & is.null(options_list[[option@dest]]) ) {
                 options_list[[option@dest]] <- option@default  
             }
         }
     }
-    if(options_list[["help"]] & print_help_and_exit) {
-        print_help(object)
-        if(interactive()) stop("help requested") else quit(status=1) 
+    if (convert_hyphens_to_underscores) {
+        names(options_list) <- gsub("-", "_", names(options_list))
+    }
+    if (any(grepl("^help$", names(options_list)))) {
+        if(options_list[["help"]] && print_help_and_exit) {
+            print_help(object)
+            if(interactive()) 
+                stop("help requested")
+            else
+                quit(status=0) 
+        }
     }
     if (length(arguments_positional) < min(positional_arguments)) {
       stop(sprintf("required at least %g positional arguments, got %g",
@@ -491,6 +550,13 @@ parse_args <- function(object, args = commandArgs(trailingOnly = TRUE),
         return(options_list)
     }
 }
+#' @rdname parse_args
+#' @export
+parse_args2 <- function(object, args = commandArgs(trailingOnly = TRUE),
+                        print_help_and_exit = TRUE) {
+    parse_args(object, args = args, print_help_and_exit = print_help_and_exit,
+               positional_arguments = TRUE, convert_hyphens_to_underscores = TRUE)
+}
 
 # Tells me whether a string is a valid option
 .is_option_string <- function(argument, object) {
@@ -511,7 +577,7 @@ parse_args <- function(object, args = commandArgs(trailingOnly = TRUE),
         if(grepl("=", argument)) {
             return(FALSE)
         } else {
-            for (ii in seq(along = object@options)) {
+            for (ii in seq_along(object@options)) {
                 option <- object@options[[ii]]
                 if(option@long_flag == argument)
                     return(option@action == "store") 
@@ -519,7 +585,7 @@ parse_args <- function(object, args = commandArgs(trailingOnly = TRUE),
         }
     } else { # is a short flag
         last_flag <- tail(.expand_short_option(argument), 1)
-        for (ii in seq(along = object@options)) {
+        for (ii in seq_along(object@options)) {
             option <- object@options[[ii]]
             if(!is.na(option@short_flag) && option@short_flag == last_flag)
                 return(option@action == "store") 
@@ -531,14 +597,14 @@ parse_args <- function(object, args = commandArgs(trailingOnly = TRUE),
 .is_short_flag <- function(argument) { return(grepl("^-[^-]", argument)) }
 .get_long_options <- function(object) {
     long_options <- vector("character")
-    for(ii in seq(along = object@options)) {
+    for(ii in seq_along(object@options)) {
         long_options <- c(long_options, object@options[[ii]]@long_flag)
     }
     return(long_options)
 }
 .get_short_options <- function(object) {
     short_options <- vector("character")
-    for(ii in seq(along = object@options)) {
+    for(ii in seq_along(object@options)) {
         short_options <- c(short_options, object@options[[ii]]@short_flag)
     }
     return(short_options)
@@ -561,7 +627,7 @@ parse_args <- function(object, args = commandArgs(trailingOnly = TRUE),
 .get_option_strings_and_n_arguments <- function(object) {
     option_strings <- vector("character")
     n_arguments <- vector("numeric")
-    for (ii in seq(along = object@options)) {
+    for (ii in seq_along(object@options)) {
         option <- object@options[[ii]]
         option_strings <- c(option_strings, option@short_flag)
         option_strings <- c(option_strings, option@long_flag)
@@ -585,3 +651,4 @@ parse_args <- function(object, args = commandArgs(trailingOnly = TRUE),
     }
     return( c( long_flag, short_flag, argument, object@type, object@help) )
 }
+
